#include <qapplication.h>
#include <qdesktopwidget.h>
#include <qinputdialog.h>
#include <qfiledialog.h>
#include <qfontmetrics.h>
#include <qlayout.h>
#include <qlistbox.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qstringlist.h>

#include "stringlistedit.h"

StringListEdit::StringListEdit(const QStringList &list, QWidget *parent)
    : QDialog(parent), ask(false), duplicatesOk(false)
{
    addCaption = tr("Add String");
    addLabel = tr("String:");
    editCaption = tr("Edit String");
    editLabel = addLabel;

    QHBoxLayout *hbox = new QHBoxLayout(this, 5, 5);
    QVBoxLayout *vbox = new QVBoxLayout;
    listBox = new QListBox(this);
    connect(listBox, SIGNAL(currentChanged(QListBoxItem*)),
            this, SLOT(updateButtons()));
    hbox->addWidget(listBox, 1);
    QPushButton *button = new QPushButton(tr("&Add..."), this);
    connect(button, SIGNAL(clicked()), this, SLOT(addString()));
    vbox->addWidget(button);
    editButton = new QPushButton(tr("&Edit..."), this);
    connect(editButton, SIGNAL(clicked()), this, SLOT(editString()));
    vbox->addWidget(editButton);
    removeButton = new QPushButton(tr("&Remove..."), this);
    connect(removeButton, SIGNAL(clicked()), this, SLOT(removeString()));
    vbox->addWidget(removeButton);
    upButton = new QPushButton(tr("&Up"), this);
    connect(upButton, SIGNAL(clicked()), this, SLOT(moveUp()));
    vbox->addWidget(upButton);
    downButton = new QPushButton(tr("&Down"), this);
    connect(downButton, SIGNAL(clicked()), this, SLOT(moveDown()));
    vbox->addWidget(downButton);
    vbox->addStretch(1);
    button = new QPushButton(tr("OK"), this);
    connect(button, SIGNAL(clicked()), this, SLOT(accept()));
    vbox->addWidget(button);
    button = new QPushButton(tr("Cancel"), this);
    connect(button, SIGNAL(clicked()), this, SLOT(reject()));
    vbox->addWidget(button);
    hbox->addLayout(vbox);

    setList(list);
}

void StringListEdit::setTexts(
        const QString &addCaption, const QString &addLabel,
        const QString &editCaption, const QString &editLabel)
{
    this->addCaption = addCaption;
    this->addLabel = addLabel;
    this->editCaption = editCaption;
    this->editLabel = editLabel.isEmpty() ? addLabel : editLabel;
}

void StringListEdit::setList(const QStringList &list)
{
    listBox->clear();
    listBox->insertStringList(list);
    QFontMetrics fm(listBox->font());
    int width = 0;
    for (uint i = 0; i < list.count(); ++i) {
        int w = fm.width(list[i]);
        if (w > width)
            width = w;
    }
    if (listBox->verticalScrollBar())
        width += listBox->verticalScrollBar()->width();
    listBox->setMinimumWidth(
        QMIN(width, qApp->desktop()->screenGeometry().width() * 4 / 5));
    updateButtons();
}

QStringList StringListEdit::list() const
{
    QStringList lst;
    for (uint i = 0; i < listBox->count(); ++i)
        lst.append(listBox->text(i));
    return lst;
}

void StringListEdit::addString()
{
    bool ok;
    QString string = QInputDialog::getText(addCaption, addLabel,
                            QLineEdit::Normal, listBox->currentText(),
                            &ok, this);
    if (ok && !string.isEmpty()) {
        if (!duplicatesOk && listBox->findItem(string,
                                               CaseSensitive | ExactMatch))
            return;
        listBox->insertItem(string);
        listBox->setCurrentItem(listBox->count() - 1);
        listBox->ensureCurrentVisible();
        updateButtons();
    }
}

void StringListEdit::editString()
{
    bool ok;
    QString original = listBox->currentText();
    if (!original.isEmpty()) {
        QString string = QInputDialog::getText(editCaption, editLabel,
                                QLineEdit::Normal, original, &ok, this);
        if (ok && !string.isEmpty()) {
            if (!duplicatesOk && listBox->findItem(string,
                                                   CaseSensitive | ExactMatch))
                return;
            listBox->changeItem(string, listBox->currentItem());
            updateButtons();
        }
    }
    else
        addString();
}

void StringListEdit::removeString()
{
    QString original = listBox->currentText();
    if (original.isEmpty() ||
        (ask && QMessageBox::question(this, tr("Remove"),
                        tr("Remove '%1'?").arg(original),
                        QMessageBox::Yes|QMessageBox::Default,
                        QMessageBox::No|QMessageBox::Escape) ==
                QMessageBox::No))
        return;
    listBox->removeItem(listBox->currentItem());
    updateButtons();
}

void StringListEdit::moveUp()
{
    int i = listBox->currentItem();
    if (i > 0) {
        QString temp = listBox->text(i - 1);
        listBox->changeItem(listBox->text(i), i - 1);
        listBox->changeItem(temp, i);
        listBox->setCurrentItem(i - 1);
        updateButtons();
    }
}

void StringListEdit::moveDown()
{
    uint i = listBox->currentItem();
    if (i < listBox->count() - 1) {
        QString temp = listBox->text(i + 1);
        listBox->changeItem(listBox->text(i), i + 1);
        listBox->changeItem(temp, i);
        listBox->setCurrentItem(i + 1);
        updateButtons();
    }
}

void StringListEdit::updateButtons()
{
    bool hasItems = (bool)listBox->count();
    editButton->setEnabled(hasItems);
    removeButton->setEnabled(hasItems);
    int i = listBox->currentItem();
    upButton->setEnabled(hasItems && i > 0);
    downButton->setEnabled(hasItems && i < (int)listBox->count() - 1);
}

void FileListEdit::addString()
{
    QString filename = QFileDialog::getOpenFileName(
                            path, filter, this, "", addCaption);
    if (!filename.isEmpty()) {
        if (!duplicatesOk && listBox->findItem(filename,
                                               CaseSensitive | ExactMatch))
            return;
        listBox->insertItem(filename);
        listBox->setCurrentItem(listBox->count() - 1);
        listBox->ensureCurrentVisible();
        path = filename;
        updateButtons();
    }
}

void FileListEdit::editString()
{
    QString filename = QFileDialog::getOpenFileName(
                            listBox->currentText(), filter,
                            this, "", editCaption);
    if (!filename.isEmpty()) {
        if (!duplicatesOk && listBox->findItem(filename,
                                               CaseSensitive | ExactMatch))
            return;
        listBox->changeItem(filename, listBox->currentItem());
        path = filename;
        updateButtons();
    }
}
